//****************************************************************************
//	
//	Copyright 2017 by WinSystems Inc.
//
//****************************************************************************
//
//	Name	 : pcmcanDLL.h
//
//	Project	 : PCMCAN Windows DLL
//
//	Author	 : Paul DeMetrotion
//
//****************************************************************************
//
//	  Date		  Rev	                Description
//	--------    -------	   ---------------------------------------------
//	05/22/17	  2.0		Original Release of DLL	
//	06/23/17	  2.1		Driver Improvements	
//
//****************************************************************************

#ifndef _PCMCAN_DLL_H_
  #define _PCMCAN_DLL_H_

#if defined DLL_EXPORT
#define DECLDIR __declspec(dllexport)
#else
#define DECLDIR __declspec(dllimport)
#endif

#define DLL_DEBUG

extern "C"
{
    DECLDIR int InitializeSession();
    DECLDIR int StartChannel(int channel, unsigned long bitRate);
    DECLDIR int ResetChannel(int channel);
    DECLDIR int SetBitRate(int channel, unsigned long bitRate);
    DECLDIR int WriteDatagram(int channel, unsigned long msgID, unsigned int msgType, int msgLen, unsigned int *data);
    DECLDIR int ReadDatagram(int channel, unsigned long *msgID, unsigned int *msgType, int *msgLen, unsigned int *data);
    DECLDIR int SetHwFilter(int channel, int mode, long acceptanceCodes, long acceptanceMasks);
    DECLDIR int ClearHwFilter(int channel);
    DECLDIR int SetSwFilter(int channel, unsigned int msgType, long lowIDRange, long highIDRange);
    DECLDIR int ClearSwFilter(int channel);
    DECLDIR int ReadStatus(int channel, unsigned int *status_reg, unsigned int *arbit_lost_capture_reg, unsigned int *error_code_capture_reg);
    DECLDIR int EnableInterrupts(int channel, unsigned int irqMask);
    DECLDIR int GetErrorCounters(int channel, unsigned int *txErrorCounter, unsigned int *rxErrorCounter);
    DECLDIR int CloseSession();
}

typedef enum {
    SUCCESS = 0,
    DRIVER_ERROR,
    INVALID_HANDLE,
    INVALID_PARAMETER,
    XMIT_BUFFER_FULL,
    RCV_BUFFER_EMPTY,
    CAN_BUS_OFF
} ErrorCodes;

typedef enum {
    STANDARD = 0,
    EXTENDED
} FrameFormatBit;

typedef enum {
    DATA = 0,
    REMOTE
} RTRBit;

typedef enum {
    SFF_DATA = 0,
    SFF_RTR,
    EFF_DATA,   
    EFF_RTR,
    MAX_MSG_TYPES
} MsgType;

typedef enum {
    DUAL = 0,
    SINGLE
} FilterMode;

typedef enum {
    ALL_IRQ_DISABLE = 0x00,
    RECEIVE_IRQ_ENBL = 0x01,
    TRANSMIT_IRQ_ENBL = 0x02,
    ERR_WARNING_IRQ_ENBL = 0x04,
    DATA_OVERRUN_IRQ_ENBL = 0x08,
    WAKE_UP_IRQ_ENBL = 0x10,
    ERROR_PASSIVE_IRQ_ENBL = 0x20,
    ARB_LOST_IRQ_ENBL = 0x40,
    BUS_ERROR_IRQ_ENBL = 0x80,
    ALL_IRQ_ENABLE = 0xFF
} InterruptMask;

#endif