//****************************************************************************
//	
//	Copyright 2017 by WinSystems Inc.
//
//****************************************************************************
//
//	Name	 : send.cpp
//
//	Project	 : PCMCAN Windows Console Application
//
//	Author	 : Paul DeMetrotion
//
//****************************************************************************
//
//	  Date		  Rev	                Description
//	--------    -------	   ---------------------------------------------
//	06/23/17	  1.0		Original Release	
//
//****************************************************************************

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>  
#include <windows.h>
#include "pcmcanDLL.h"

#define MAJOR_VER   1
#define MINOR_VER   0

#define CHANNEL     1
#define BITRATE     500000

#define IRQ         'Y'  // Y or N

// tramsit IDs
#define SFF_ID1     0x400
#define SFF_ID2     0x0ab
#define SFF_ID3     0x123
#define EFF_ID1     0x1f000000
#define EFF_ID2     0x00aabbcc
#define EFF_ID3     0x12345678

#define TBS_BIT     0x04

#if IRQ == 'N'
void waitForXmit(void)
{
    int dllReturn = 0;
    unsigned int status_reg;
    unsigned int arbit_lost_capture_reg;
    unsigned int error_code_capture_reg;

    do
    {
        dllReturn = ReadStatus(CHANNEL, &status_reg, &arbit_lost_capture_reg, &error_code_capture_reg);

        if (dllReturn)
        {
            printf("Error reading status\n");
            exit(dllReturn);
        }

        Sleep(1);
    } while (!(status_reg & TBS_BIT));
}
#endif

int _tmain(int argc, _TCHAR* argv[])
{
    int dllReturn = 0;
    unsigned int data1[] = { 0 };
    unsigned int data2[] = { 0xa5, 0xc3 };
    unsigned int data4[] = { 0x01, 0x02, 0x03, 0x04 };
    unsigned int data6[] = { 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF };
    unsigned int data8[] = { 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88 };
    long pckts_sent = 0;

    printf("PCM-CAN Application : send\n");
    printf("Version %d.%d\n\n", MAJOR_VER, MINOR_VER);

    dllReturn = InitializeSession();

    if (dllReturn)
    {
        printf("Error initializing session\n");
        exit(dllReturn);
    }

    dllReturn = StartChannel(CHANNEL, BITRATE);
    
    if (dllReturn)
    {
        printf("Error starting channel %d\n", CHANNEL);
        exit(dllReturn);
    }

#if IRQ == 'N'
    dllReturn = EnableInterrupts(CHANNEL, ALL_IRQ_ENABLE & ~TRANSMIT_IRQ_ENBL);

    if (dllReturn)
    {
        printf("Error disabling transmit interrupt\n");
        exit(dllReturn);
    }
#else
    dllReturn = EnableInterrupts(CHANNEL, ALL_IRQ_ENABLE);

    if (dllReturn)
    {
        printf("Error enabling all interrupts\n");
        exit(dllReturn);
    }
#endif

    printf("Sending CAN packets ...\n");

    while (1)
    {
        Sleep(1);

        do {
            dllReturn = WriteDatagram(CHANNEL, EFF_ID1, EFF_DATA, sizeof(data4) / sizeof(unsigned int), data4);

            if (dllReturn != SUCCESS && dllReturn != XMIT_BUFFER_FULL)
            {
                printf("Error sending message\n");
                CloseSession();
                exit(dllReturn);
            }
        } while (dllReturn);

#if IRQ == 'N'
        waitForXmit();
#endif

        Sleep(1);

        do {
            dllReturn = WriteDatagram(CHANNEL, SFF_ID1, SFF_DATA, sizeof(data8) / sizeof(unsigned int), data8);

            if (dllReturn != SUCCESS && dllReturn != XMIT_BUFFER_FULL)
            {
                printf("Error sending message\n");
                CloseSession();
                exit(dllReturn);
            }
        } while (dllReturn);

#if IRQ == 'N'
        waitForXmit();
#endif

        Sleep(1);

        do {
            dllReturn = WriteDatagram(CHANNEL, EFF_ID2, EFF_RTR, 0, NULL);


            if (dllReturn != SUCCESS && dllReturn != XMIT_BUFFER_FULL)
            {
                printf("Error sending message\n");
                CloseSession();
                exit(dllReturn);
            }
        } while (dllReturn);

#if IRQ == 'N'
        waitForXmit();
#endif

        Sleep(1);

        do {
            dllReturn = WriteDatagram(CHANNEL, SFF_ID2, SFF_DATA, sizeof(data6) / sizeof(unsigned int), data6);

            if (dllReturn != SUCCESS && dllReturn != XMIT_BUFFER_FULL)
            {
                printf("Error sending message\n");
                CloseSession();
                exit(dllReturn);
            }
        } while (dllReturn);

#if IRQ == 'N'
        waitForXmit();
#endif

        Sleep(1);

        do {
            dllReturn = WriteDatagram(CHANNEL, EFF_ID3, EFF_DATA, sizeof(data2) / sizeof(unsigned int), data2);

            if (dllReturn != SUCCESS && dllReturn != XMIT_BUFFER_FULL)
            {
                printf("Error sending message\n");
                CloseSession();
                exit(dllReturn);
            }
        } while (dllReturn);

#if IRQ == 'N'
        waitForXmit();
#endif

        Sleep(1);

        do {
            dllReturn = WriteDatagram(CHANNEL, SFF_ID3, SFF_RTR, 0, NULL);

            if (dllReturn != SUCCESS && dllReturn != XMIT_BUFFER_FULL)
            {
                printf("Error sending message\n");
                CloseSession();
                exit(dllReturn);
            }
        } while (dllReturn);

#if IRQ == 'N'
        waitForXmit();
#endif

        pckts_sent += 6;

        printf("Packets sent: %ld\n", pckts_sent);
    }
    
    CloseSession();

    return dllReturn;
}
